function val = dh2norm(SYS)
// computes the H2 norm of a discrete-time state-space system

// algorithm: see Zhou, K.; Doyle, J. & Glover, K., "Robust and
// Optimal Control", Prentice-Hall, 1996

  // check inputs
  if typeof(SYS)~="state-space" then
    error("SYS is no state-space system");
  end
  if typeof(SYS.D)~="constant" then
    error("SYS.D is no constant");
  end
  if SYS.dt~="d" then
    error("SYS is no discrete-time system");
  end

  // compute gramian
  [A,B,C,D] = abcd(SYS);
  X = cplxlyap(A,-C'*C);
  
  // compute norm
  val = real(sqrt(trace(B'*X*B+D'*D)));

endfunction
