function [SMP,A,Atilde,deg] = smpafact(SYSH,L)
// factorizes SYSH=SMP*A where SMP is strictly minimum-phase and A is a FIR inner function of degree deg<=L; Atilde=A(1/z)'/z^deg

// algorithm: this is a special form of Silvermans Structure Algorithm; see e.g. Silverman,
// "Inversion of Multivariable Linear Systems", IEEE Trans. Autom. Control, Vol. 14, No. 3, 1969

  // initialize algorithm
  [q,p] = size(SYSH.D);
  SMP = SYSH;
  deg = 0;
  A = syslin("d",[],[],[],eye(p,p));
  delay = syslin("d",0,1,1,0);

  // compute SMP and A
  [U,S,V,rk] = svd(SMP.D);
  while rk<p do
    SMP = SMP*V;
    SMP = syslin("d",SMP.A,[SMP.B(:,1:rk) SMP.A*SMP.B(:,rk+1:$)],SMP.C,[SMP.D(:,1:rk) SMP.C*SMP.B(:,rk+1:$)]);
    A = [eye(rk,rk) zeros(rk,p-rk) ; zeros(p-rk,rk) delay*eye(p-rk,p-rk)]*V'*A;
    deg = deg+1;
    if deg>L then
      error(sprintf("Could not factorize"));
    end
    [U,S,V,rk] = svd(SMP.D);
  end

  // compute Atilde
  Atilde = syslin("d",[],[],[],A.D');
  if deg>=1 then
    Atilde = (delay*eye(p,p))*Atilde + A.B'*A.C';
  end
  for k=2:deg do
    Atilde = (delay*eye(p,p))*Atilde + A.B'*A.A'^(k-1)*A.C';
  end

endfunction